package br.usp.ime.algorithms;

import java.util.ArrayList;
import java.util.List;
import java.util.PriorityQueue;

import br.usp.ime.evolvinggraph.Edge;
import br.usp.ime.evolvinggraph.EvolvingGraph;
import br.usp.ime.evolvinggraph.Interval;
import br.usp.ime.evolvinggraph.Journey;
import br.usp.ime.evolvinggraph.Node;
import br.usp.ime.journeyclass.JourneyClass;
import br.usp.ime.priorityqueueelement.FastestElement;

public class FastestAlgorithmAlt implements Algorithm {

	//@Override
	public List<Journey> calculate(EvolvingGraph graph, Node s) {
		return calculate(graph, s, 0);
	}
	
	public List<Journey> calculate(EvolvingGraph graph, Node s, int initTime) {
		PriorityQueue<FastestElement> queue = new PriorityQueue<FastestElement>();
		List<Journey> journeyCollection = new ArrayList<Journey>(graph.getNumOfNodes());
		
		for (int i = 0; i < graph.getNumOfNodes(); i++){
			journeyCollection.add(i, null);
		}
		journeyCollection.set(s.getId(), new Journey());
		
		for (Edge e : s.getAdj()){
			
			for (Interval inter : e.getSchedule().getIntervals()){
				int delay;
				Journey journey = new Journey();
				// Intervalos anteriores a initTime
				if (inter.getEnd() <= initTime)
					continue;
				// Intervalos ativos em initTime
				if (inter.getStart() < initTime){
					delay = inter.getEnd()-1-initTime;
					journey.append(e, initTime);
				}
				// Intervalos futuros
				else{
					delay = inter.getEnd()-1-inter.getStart();
					journey.append(e, inter.getStart());
				}
				queue.add(new FastestElement(e.getV(), journey, delay));
			}
		}
		
		while (queue.isEmpty() == false){
			//
			FastestElement element = queue.remove();
			Node u = element.getU();
			int t = element.getArrival();
			int c = element.getCost();
			if (journeyCollection.get(u.getId()) == null){
				journeyCollection.set(u.getId(), element.getJourney());
			}
			for (Edge e : u.getAdj()){
				Node v = e.getV();
				Interval departureWindow = e.getNextDepartureInterval(t);
				if (departureWindow != null){
					int departureTime = departureWindow.getStart();
					int delay = t + element.getDelay() - departureTime;
					if (delay < departureWindow.getEnd() - departureWindow.getStart())
						delay = departureWindow.getEnd() - departureWindow.getStart();
					if (delay < 0)
						delay = 0;
					Journey journey = new Journey(element.getJourney());
					System.out.println(journey);
					System.out.println(delay + " = " + t + "+" + element.getDelay() + "-" + departureTime);
					System.out.println(e.getU().getId() + "-" + e.getV().getId());
					// Atrasar prefixo
					int prevDelay = departureTime-t;
					if (prevDelay > element.getDelay())
						prevDelay = element.getDelay();
					journey.delay(prevDelay);
					journey.append(e, departureTime);
					JourneyClass journeyClass = new JourneyClass();
					journeyClass.setJourney(journey);
					journeyClass.setDelay(delay);
					
					Journey prev = journeyCollection.get(v.getId());
					int newCost = departureTime - t + c + e.getCost();
					if (prev == null || prev.getTransitTime() > newCost)
						queue.add(new FastestElement(v, journeyClass));
				}
			}
		}
		
		return journeyCollection;
	}

}
