package br.usp.ime.algorithms;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import br.usp.ime.evolvinggraph.Edge;
import br.usp.ime.evolvinggraph.EvolvingGraph;
import br.usp.ime.evolvinggraph.Journey;
import br.usp.ime.evolvinggraph.Node;

public class ShortestAlgorithm implements Algorithm {

	Vector<Edge> eMin;
	Vector<Integer> tMin;
	
	private boolean edgeAndScheduleSelection(EvolvingGraph graph, Vector<Integer> tLBD){
		Vector<Integer> arrivalTime = new Vector<Integer>();
		boolean improved = false;
		
		eMin = new Vector<Edge>(graph.getNumOfNodes());
		tMin = new Vector<Integer>(graph.getNumOfNodes());
		
		for (int i = 0; i < graph.getNumOfNodes(); i++){
			eMin.add(i, null);
			tMin.add(i, Integer.MAX_VALUE);
			arrivalTime.add(i, tLBD.get(i));
		}
		for (Node node : graph.getNodes()){
			if (tLBD.get(node.getId()) != Integer.MAX_VALUE){
				for (Edge edge : node.getAdj()){
					Integer newTime = edge.getNextDeparture(tLBD.get(node.getId()));
					if (newTime != null) {
						Node v = edge.getV();
						if (newTime + edge.getCost() < arrivalTime.get(v.getId())){
							eMin.set(v.getId(), edge);
							tMin.set(v.getId(), newTime);
							arrivalTime.set(v.getId(), newTime + edge.getCost());
							improved = true;
						}
					}
				}
			}
		}
		return improved;
	}
	
	//@Override
	public List<Journey> calculate(EvolvingGraph graph, Node s) {
		return calculate(graph, s, 0);
	}
	
	public List<Journey> calculate(EvolvingGraph graph, Node s, int initTime) {
		List<Journey> shortestJourney = new ArrayList<Journey>();
		List<Journey> currentJourney = new ArrayList<Journey>();
		Vector<Integer> tLBD = new Vector<Integer>(graph.getNumOfNodes());
		
		for (int i = 0; i < graph.getNumOfNodes(); i++){
			tLBD.add(i, Integer.MAX_VALUE);
			currentJourney.add(i, new Journey());
			shortestJourney.add(i, null);
		}
		tLBD.set(s.getId(), initTime);
		currentJourney.set(s.getId(), new Journey());
		shortestJourney.set(s.getId(), new Journey());
		
		int numSteps = 0;
		int unreachableNodes = graph.getNumOfNodes() - 1;
		
		while (numSteps < graph.getNumOfNodes() && unreachableNodes > 0){
			List<Journey> lastJourney = new ArrayList<Journey>();
			if (edgeAndScheduleSelection(graph, tLBD) == false)
				break;
			
			for (int i = 0; i < graph.getNumOfNodes(); i++){
				lastJourney.addAll(currentJourney);
			}
			
			for (Node node : graph.getNodes()){
				Edge edge = eMin.get(node.getId());
				if (edge != null){
					int uId = edge.getU().getId();
					int vId = edge.getV().getId();
					int t = tMin.get(vId);
					Journey journey = new Journey(lastJourney.get(uId), edge, t);
					currentJourney.set(vId, journey);
					if (tLBD.get(vId) == Integer.MAX_VALUE){
						shortestJourney.set(vId, journey);
						unreachableNodes--;
					}
					tLBD.set(vId, t + edge.getCost());
				}
			}
			numSteps = numSteps + 1;
		}
		
		return shortestJourney;
	}

}
