package br.usp.ime.algorithms;

import java.util.ArrayList;
import java.util.List;
import java.util.PriorityQueue;

import br.usp.ime.evolvinggraph.Edge;
import br.usp.ime.evolvinggraph.EvolvingGraph;
import br.usp.ime.evolvinggraph.Interval;
import br.usp.ime.evolvinggraph.Journey;
import br.usp.ime.evolvinggraph.Node;
import br.usp.ime.journeyclass.JourneyClass;
import br.usp.ime.priorityqueueelement.FastestElement;

public class WeightedAlgorithm implements Algorithm {
	int arrivalTimeWeight;
	int hopcountWeight;
	int transitTimeWeight;
	
	public WeightedAlgorithm(int arrivalTimeWeight, 
			int hopcountWeight) {
		this.arrivalTimeWeight = arrivalTimeWeight;
		this.hopcountWeight = hopcountWeight;
		this.transitTimeWeight = 0;
	}
	
	public WeightedAlgorithm(int arrivalTimeWeight, 
			int hopcountWeight, int transitTimeWeight) {
		this.arrivalTimeWeight = arrivalTimeWeight;
		this.hopcountWeight = hopcountWeight;
		this.transitTimeWeight = transitTimeWeight;
	}
	
	public void setArrivalTimeWeight(int weight){
		arrivalTimeWeight = weight;
	}
	
	public void setHopCountWeight(int weight){
		hopcountWeight = weight;
	}
	
	public void setTransitTimeWeight(int weight){
		transitTimeWeight = weight;
	}

	//@Override
	public List<Journey> calculate(EvolvingGraph graph, Node s) {
		return calculate(graph, s, 0);
	}
	
	public List<Journey> calculate(EvolvingGraph graph, Node s, int initTime) {
		PriorityQueue<FastestElement> queue = new PriorityQueue<FastestElement>();
		List<Journey> journeyCollection = new ArrayList<Journey>(graph.getNumOfNodes());
		
		for (int i = 0; i < graph.getNumOfNodes(); i++){
			journeyCollection.add(i, null);
		}
		journeyCollection.set(s.getId(), new Journey());
		
		for (Edge e : s.getAdj()){
			
			for (Interval inter : e.getSchedule().getIntervals()){
				int delay;
				Journey journey = new Journey();
				if (inter.getEnd() <= initTime)
					continue;
				if (inter.getStart() < initTime){
					delay = inter.getEnd()-initTime;
					journey.append(e, initTime);
				}
				else{
					delay = inter.getEnd()-inter.getStart();
					journey.append(e, inter.getStart());
				}
				queue.add(new FastestElement(e.getV(), journey, delay));
			}
		}
		
		while (queue.isEmpty() == false){
			//
			FastestElement element = queue.remove();
			Node u = element.getU();
			int t = element.getArrival();
			int c = element.getCost();
			if (journeyCollection.get(u.getId()) == null){
				journeyCollection.set(u.getId(), element.getJourney());
			}
			for (Edge e : u.getAdj()){
				Node v = e.getV();
				Interval departureWindow = e.getNextDepartureInterval(t);
				if (departureWindow != null){
					int departureTime = departureWindow.getStart();
					Journey journey = new Journey(element.getJourney(), e, departureTime);
					JourneyClass journeyClass = new JourneyClass();
					int delay = t + element.getDelay() - departureTime;
					if (delay < 0)
						delay = 0;
					journeyClass.setJourney(journey);
					journeyClass.setDelay(delay);
					
					Journey prev = journeyCollection.get(v.getId());
					int newCost = departureTime - t + c + e.getCost();
					if (prev == null || prev.getTransitTime() > newCost)
						queue.add(new FastestElement(v, journeyClass));
				}
			}
		}
		
		return journeyCollection;
	}

}
